class PostcoderRooftop{
  
  constructor(config) {
    this.config = config
    this.init()
  }

  init = () => {

    this.endpointGB = 'https://ws.postcoder.com/pcw/' + this.config.apikey + '/pafaddressbase/'
    this.endpointIE = 'https://ws.postcoder.com/pcw/' + this.config.apikey + '/address/ie/'
    this.addresses = {}

    this.addresslines = 4

    this.searchbutton = document.querySelector(this.config.searchbutton)
    this.searchterminput = document.querySelector(this.config.searchterm)
    this.addressselectioncontainer = document.querySelector(this.config.addressselectioncontainer)


    //hide the no result message
    this.noresultmessage = document.querySelector(this.config.noresultmessage)
    this.noresultmessage.style.display = 'none'

    // add an empty container to show the selected address
    this.addresscontainer = document.createElement('div')
    this.addresscontainer.classList.add('address')
    this.addressselectioncontainer.parentNode.insertBefore(this.addresscontainer, this.addressselectioncontainer.nextSibling)

    // create an empty select and place it hidden in the page
    this.select = document.createElement('select')
    this.select.style.display = 'none'
    this.addressselectioncontainer.appendChild(this.select)

    // create and append the top 'select an address' option.
    let option = document.createElement('option')
    option.text = 'Select an address'
    this.select.appendChild(option)

    // add select change event listener to the dropdown
    this.select.addEventListener('change', this.handleSelectChange )

    // add click event listener to the search button, to call the search function
    this.searchbutton.addEventListener('click', this.search)

    // add 'enter' event listener to the search input, to call the search function
    this.searchterminput.addEventListener('keyup', event =>{
      if (event.keyCode === 13) {
        event.preventDefault()
        this.search()
      }
    })

    // initalise an OpenLayers map and marker, on London
    window.onload = (event) => {
  		
    	const london = ol.proj.fromLonLat([-0.12755, 51.507222])

    	this.view = new ol.View({
  			center: london,
  			zoom: 9,
		  })

    	this.map = new ol.Map({
    		target: this.config.mapcontainer,
    		controls: ol.control.defaults({ attribution: false }),
    		layers: [
	      		new ol.layer.Tile({
    	    		source: new ol.source.OSM()
      			})
    		],
    		view: this.view,
  		})

      this.markerlayer = new ol.layer.Vector({
  			source: new ol.source.Vector(),
  				style: new ol.style.Style({
    				image: new ol.style.Icon({
      				anchor: [0.5, 1],
      				src: 'https://postcoder.com/images/svg/map-pin.svg'
    			})
  			})
		  })

		  this.map.addLayer(this.markerlayer)

		  this.marker = new ol.Feature(new ol.geom.Point(london))
		  this.markerlayer.getSource().addFeature(this.marker)

    }
  
  }

  handleSelectChange = event => {

      // check to see if this selection was to load the next page of results
      if( this.select.value.charAt(0) === 'p'){
      
        // search the next page of results
        let page = this.select.value.substring(1)
        this.search(event, page)

      }else{   

        this.selectAddress( this.select.value )

      }     
  }

  search = (event, page = 0) => {

    let searchterm = encodeURIComponent(this.searchterminput.value)

    //hide the no result message
    this.noresultmessage.style.display = 'none'

    // hide the select
    this.select.style.display = 'none'

    //determine the country
    if(typeof this.config.countrycode !== 'undefined' && this.config.countrycode !== ''){
      // defined in config
      var country = this.config.countrycode
    }else{
      // get from dropdown
      var country = document.querySelector(this.config.country).value
    }

    // if page 0, remove any previous options
    if(page === 0){
      while (this.select.options.length > 1) {                
        this.select.remove(1)
      }
    }else{
      // remove the bottom 'next page' option
      this.select.remove(this.select.options.length - 1)
    }

    // different endpoint for GB and IE
    if( country == 'IE'){
    	var url = this.endpointIE + searchterm + '?lines='+this.addresslines + '&addtags=latitude,longitude&exclude=organisation,country&page=' + page
    }else{
    	var url = this.endpointGB + searchterm + '?lines='+this.addresslines + '&exclude=organisation,country&page=' + page
    }
   	

    // fetch the json formatted result from Postcoder and pass it to processResult
    fetch(url)
      .then( response => {
        if (!response.ok) { 
          throw response 
        }
        return response.json()
      })
      .then( json => {
        this.processResult(json)
      })
      .catch( err => {

        if(typeof err.text === 'function'){
          err.text().then( errorMessage => {
            console.log('Postcoder request error ' + err.status + ' : ' + errorMessage)
          })
        }else{
          console.log(err)
        }
    })
    
  }

  processResult = addresses =>{

    this.addresses = addresses

    //if zero results, ask for manual address entry
    if( addresses.length === 0 ){
      
      //show the no result message
      this.noresultmessage.style.display = 'block'

    }
    // if one result, populate the form
    else if(addresses.length === 1){

      this.selectAddress(0)

    }
    // multiple addresses, show drop down to select from
    else{

      // create and append the address options, using the summaryline of the returned addresses
      for (let i = 0; i < addresses.length; i++) {
        let option = document.createElement('option')
        option.value = i
        option.text = addresses[i].summaryline
        this.select.appendChild(option)
      }

      // if there are morevalues, add paging
      if(this.addresses[this.addresses.length - 1]['morevalues'] === 'true'){

        // add a 'load next page' option
        let option = document.createElement('option')
        option.value = 'p' + this.addresses[this.addresses.length - 1]['nextpage']

        // last address contains total number of results, show the user how many are remaining
        let remainingresults = this.addresses[this.addresses.length - 1]['totalresults'] - this.select.options.length + 1

        option.text = 'Load more results, ' + remainingresults + ' of ' + this.addresses[this.addresses.length - 1]['totalresults'] + ' remaining'
        this.select.appendChild(option)

      }
      
      // ensure the select is visible
      this.select.style.display = 'block'

    }

  }

  selectAddress = i =>{

    // get the selected address
    let address = this.addresses[i]

    let fields = ['organisation','addressline1','addressline2','addressline3','addressline4','posttown','county','postcode']

    let displayaddress = ''

    // display the selected adress in the address container
    for (let i = 0; i < fields.length; i++) {
      if(typeof address[fields[i]] !== 'undefined'){
     	
      	displayaddress += address[fields[i]] + '<br>'

      }
     
    }

    this.addresscontainer.innerHTML = displayaddress

    this.showMap( i )

  }


  showMap = i =>{

    // get the selected address
    let address = this.addresses[i]

    if(address.latitude && address.longitude){

    	let position = ol.proj.fromLonLat([address.longitude, address.latitude])

    	this.view.animate({
    		center: position,
    		zoom: 18,
    		duration: 2000,
  		})

    	this.marker.getGeometry().setCoordinates(position)

    }

  }

}